/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   main.c                                             :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: cchauvet <cchauvet@student.42angoulem      +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2023/02/16 15:16:14 by cchauvet          #+#    #+#             */
/*   Updated: 2023/02/24 11:04:40 by cchauvet         ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "cmd/cmd.h"
#include "libftx/libft/list.h"
#include "minishell.h"

static char	*ft_get_user_input()
{
	char	*line;
	char	*prompt;
	char	*pwd;

	pwd = get_pwd(2);
	if (pwd == NULL)
		return (NULL);
	prompt = ft_strmerger(2, pwd, "$ ");
	free(pwd);
	if (prompt == NULL)
	{
		ft_eprintf("minishell: malloc failed\n");
		return (NULL);
	}
	line = readline(prompt);
	if (line != NULL && ft_strcmp(line, "") != 0)
		add_history(line);
	free(prompt);
	return (line);
}

static int	ft_minishell(t_data *data, char *line)
{
	char	*line_clean;
	t_list	*current;
	t_cmd	*content;

	if (ft_syntax_verif(data, line))
		return (0);
	line_clean = ft_formater(line);
	if (line_clean == NULL)
		return (0);
	if (ft_cmds_parser(data, line_clean))
	{
		free(line_clean);
		return (0);
	}
	free(line_clean);
	if (ft_cmds_executor(data) == 1)
		return (1);
	current = *data->cmds;
	while (current != NULL)
	{
		content = current->content;
		if (content->own_cmd == 0 && content->pid != -1)
			waitpid(content->pid, &data->exit_code, 0);
		current = current->next;
	}
	ft_lstclear(data->cmds, ft_cmddel);
	return (0);
}

void	ft_ctrlc(int num)
{
	(void) num;

	if (*ft_get_heredoc() != -1)
	{
		close(*ft_get_heredoc());
		*ft_get_heredoc() = -1;
	}
	else
	{
		rl_replace_line("", 0);
		rl_on_new_line();
		ft_putchar_fd('\n', 1);
		rl_redisplay();
	}
}

void	ft_quit(int num)
{
	(void) num;
	
	ft_printf("%c%c %c%c", 0, 0, 0, 0);
}

int	main(int ac, char **av, char **env)
{
	t_data	data;
	char	*line;

	(void) ac;
	(void) av;
	signal(SIGINT, ft_ctrlc);
	signal(SIGQUIT, ft_quit);
	data.exit_code = 0;
	data.exit_code_str = NULL;
	ft_gen_exit_code(&data);
	data.cmds = malloc(sizeof(t_cmd *));
	if (data.cmds == NULL)
		return (1);
	*data.cmds = NULL;
	data.env = init_env(env);
	if (data.env == NULL)
		return (1);
	line = ft_get_user_input();
	while (line != NULL)
	{
		if (ft_minishell(&data, line) == 1)
			break ;
		free(line);
		line = ft_get_user_input();
		if (line == NULL)
			break ;
	}

	free(data.exit_code_str);
	ft_lstclear(data.cmds, ft_cmddel);
	free(data.cmds);
	ft_lstclear(data.env, env_del);
	free(data.env);
	return (data.exit_code);
}
